	list p=16f88
	include "p16f88.inc"
	include "macros.inc"
	radix dec
	errorlevel -302
	errorlevel -306

        expand        
        ;noexpand


  __CONFIG 0x2007, _BODEN_OFF & _CP_OFF & _CPD_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLR_OFF & _INTRC_IO

        
;;; Pin assignments

;;;; Receive into PIC from the PC (while testing use RB0 on pin 6)
;;     later use RA5/Vpp/MCLR* on pin 4
        
        ;; for testing, I used RA7 on pin 15 
RXPORT    equ PORTA
RXPIN     equ 7
RXTRIS    equ TRISA

        ;; transmit serial data on RA6  (pin 16)
TXPORT    equ PORTA
TXPIN     equ 6
TXTRIS    equ TRISA

KillMicroseconds macro num
        ;; x = 256 is maximum, so  5 + 3*256  =  773   i.e. (+ 5 (* 3 256))
        ;;  is the largest allowed value for num.  The smallest value
        ;;  would be x = 1 for a value for num of  5 + 3  =  8
        ;;  Accept values less than 8 but, for them, do in-line 'nop's
        if (num > 773)
      	    Error "KillMicroseconds called with value greater than 773"
        else
           if (num < 8)
              if (num > 0)
                 nop
                 KillMicroseconds (num - 1)
              endif
           else
              movlw ((num - 5) / 3)
              call ShortDelay
           endif
        endif
    endm

        

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Start of program
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        org 0x0020
UserISR
        ;; this is the beginning of the second 32-word block.  It is where the
        ;; user's code starts.  First, with 4 words allotted to the ISR
        ;; (the real ISR jumps here), then, at 0x0024, the user's startup code.
        ;; User is allowed 4 instructions here to jump to actual ISR
        retfie
        nop
        nop
        nop
        
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        org 0x0024
UserApp        
hex2ascii:
	andlw	15		; Lower nybble only
	addwf	PCL,F		; 
	dt	"0123456789ABCDEF"

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Loader
;;;    Note, we could put some of the loader at 0x0008, since the user's code must
;;;    not touch the first 32-word block.  For now, put all the loader in high
;;;    memory.  Note, the '818 has 1K words of program flash (1024) (0x0000-0x3ff),
;;;    and the '819 has 2K words (2048) (0x0000-0x7ff).
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;ZFlashSize equ 0x0800            ; for 16F819
;ZFlashSize equ 0x0400            ; for 16F818
;ZFlashSize equ 0x2000            ; for 16F877
ZFlashSize equ 0x1000            ; for 16F88
        

        call Synchronize        ; this also gets the first command

        ;; the first command is now in W (and also in rxdata)
        ;; we are prepared only for "P" (for Program)
        IfEqLit 'P'
        goto Program
        
	pagesel	UserApp
        goto UserApp

SYNCCHAR equ 'U'
        
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Synchronize
;;;
;;;   (This could be named SynchronizeAndGetFirstCommand, but that's a bit verbose!)
;;; 
;;;   Listen to the serial port for a while to see if the host PC wants our attention.
;;;   If it does, we will (eventually) detect a SYNC character (defined above).  Since
;;;   we are bit banging the serial port and since the PIC might wake up and start
;;;   listening to the serial port in the middle of a character, we need to make sure
;;;   we choose a SYNC character that will allow synchronization.
;;;   After at least one SYNC char, return the first non-SYNC character as the
;;;   command.
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Synchronize
        clrf wiggles
        LFor 0x4000
          call WigglingWithDelay
          btfsc wiggles,3         ; did wiggles count up to at least 00001xxx
          goto GetSyncChar        ;  if so, jump out early
        LNext
	pagesel	UserApp
        goto UserApp              ;  otherwise, give up and jump to the application

GetSyncChar
        ;; ok, we got a wiggle
        clrf countL             ; read up to 256 characters looking for a SYNC
SyncLoop
        call Rx1
        sublw SYNCCHAR
        IfZ                     ; once we see at least one sync character we
        goto GetCommand         ;  are ready to look for a command
        decfsz countL, F      ; if not a sync char, have we waited long enough?
        goto SyncLoop           ; if not, keep looking
	pagesel	UserApp
        goto UserApp            ; if so, give up and run the regular application
        ;; the problem here, is that if the serial line does not *keep* wiggling,
        ;;  we will hang forever in the call to Rx1.  I guess we could add some
        ;;  time out logic to Rx1, but ignore it for now.

GetCommand
        ;; Ok, we got at least one sync char.  Keep watching serial input until
        ;;   we get a command byte (i.e. something other than the sync char).
        ;;   If we get a "P" (for "Program") then we will burn a new application
        ;;   into the Flash.  If it is any other command, it is unknown, so we
        ;;   jump to the regular application.  Upon return, the command character
        ;;   is in W and also in rxdata
        ;;
        call Rx1
        sublw SYNCCHAR
        IfZ                     ; throw away additional sync bytes
        goto GetCommand
        movf rxdata, W
        return


;;; Initialize
Initialize
        Bank0
        ;; Some of this might need to be customized for the specific
        ;;  application or class of application, particularly the
        ;;  direction of I/O pins so that, e.g. application circuit
        ;;  signals are in a safe state.

        ;; We use no interrupts, and its are disabled by default, but
        ;;  make it explicit by clearing INTCON here to disable all
        ;;  interrupts.
        clrf INTCON
        
        ;; We do not use the timer0 interrupt.  But, if we
        ;;  did, we would need to clear the T0IF flag prior to enabling
        ;;  the interrupt (and also do this before returning from its 
        ;;  int handler).
        ;bcf     INTCON, T0IF    ; turn off flag
        
        clrf PORTA
        clrf PORTB

        Bnk1
        ;; Set clock to 4 MHz
        movlw b'01100000'       ; 4 MHz
        movwf OSCCON
        
        ; turn analog/comparators off for 16LF819 and make pins digital
        movlw 0x06              ; make all PORTA pins digital
        movwf ADCON1            ;   (manual p. 82)
        
        ;; Port A direction
        clrf TRISA             ; all outputs (as much as possible)
        
        ;; Port B direction
        clrf TRISB             ; all outputs

        ;; for serial output, TXPORT,TXPIN (i.e. PORTA,1) must be output
        bcf TXTRIS, TXPIN       ; clear it to make RA1 an output

        ;; for serial input, RXPORT,RXPIN (i.e. at first PORTB,0 -- then later PORTA,5)
        ;;  must be an input
        bsf RXTRIS, RXPIN
        
        Bnk1
        ;; ** Option Register **
 	movlw B'11011101'  ; prescaler middle speed (divide by 32) and no pull-ups
	movwf OPTION_REG
        
	Bnk0
        
	;;;; init variables and queues
        clrf lastSerialIn
        clrf wiggles

;;;; Serial and timer interrupts
        ;; We are not using interrupts, but if we were, this would
        ;;   be the place to enable them.
	clrf PIR1
	clrf INTCON
        
	;; Timer1, internal, divide by 8
	;; TMR1L then increments every 8 uS with a 4 MHz crystal
	;;  and can count up to (255 * 8) = 2040 uS ~= 2 mS
	;; Consult TMR1L for times under ~ 2 ms and
	;; consult TMR1H for times between about 2 mS and 1/2 second
	movlw B'00000001'
	movwf T1CON

	Bnk1
	clrf PIE1

	Bnk0
	clrf PIR1; do it again, just in case (clear interrupt flags)

	return
;;; end of 16LF819 bootloader initialize routine
        


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Serial out -- inverted TTL serial in software at 9600 bps or 57600 bps
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

myTx1
	;; Transmit character in W to the serial port
        ;; This will be a software serial port, producing fake RS232,
        ;;  i.e. inverted TTL serial.  So, a start bit will be high
        ;;  and a stop bit will be low.

        ;; For 9600 BPS, a bit period is  1,000,000 / 9600 --> 104-1/6 uS.
        ;; For 57600 BPS, a bit period is 17.36 uS.
        ;; For the default 4 MHz clock, an instruction takes 1 uS.  So,
        ;; we need to kill about 104 uS or 17 per bit.

        ;; Byte to be transmitted is in W, put it into txdata
        movwf txdata
        call SerialZero         ; send the logic 0 start bit (a physical high)
        call SerialBit          ; send each of the 8 data bits
        call SerialBit
        call SerialBit
        call SerialBit
        call SerialBit
        call SerialBit
        call SerialBit
        call SerialBit
        call SerialOne          ; send the stop bit
        return
SerialZero
        ;; Set TXPIN high to indicate a logic zero and kill a bit period
        bsf TXPORT, TXPIN
        goto delayFullAuto
SerialOne
        ;; Set TXPIN low to indicate a logic one and kill a bit period
        nop  ; needed (btfsc instruction in SerialBit) so both paths take same time.
        bcf TXPORT, TXPIN
        goto delayFullAuto
SerialBit
        ;; Send the current right-most bit in txdata (by shifting it into the carry)
        rrf txdata, f
        btfsc STATUS, C
        goto SerialOne                  ; the bit is a one
        goto SerialZero                 ; the bit is a zero


        
KillBitPeriodFor9600
        ;; So, about 10 uS have been used up so far.  kill about 92 more,
        ;; including our return instruction.
        movlw d'29'
ShortDelay
        ;; The formula (at 4 MHz) for calling ShortDelay, including the
        ;; call and also the 'movlw x' instruction is
        ;; delay in uS  =  5 + 3x
        ;; E.g.
        ;;        movlw d'29'
        ;;        call ShortDelay
        ;;  would take 5 + 3*29  = 29 uS
        movwf delayL
ShortDelayLoop
        decfsz delayL, F
        goto ShortDelayLoop
        return
        
delayFullAuto
	movf	autobauddelay,W	; delay for about 198 uS
	goto	ShortDelay

delayHalfAuto
	bcf	STATUS,C	; Prepare shift
	rrl	autobauddelay,W	; Half the delay
	goto	ShortDelay
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Some macros to make the inverted logic easier to deal with
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        
;;; IfRxZero
        ;; perform the following instruction only if the serial input is a
        ;; serial zero bit (a high voltage)
IfRxZero macro
        btfsc RXPORT,RXPIN
	endm

;;; IfRxOne
        ;; perform the following instruction only if the serial input is a
        ;; serial one bit (a low voltage)
IfRxOne macro
        btfss RXPORT,RXPIN
	endm
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Serial in -- inverted TTL serial in software at 9600 bps or 57600 bps
;;; Use only 9600 bps to start with.
;;; 
;;; We need a routine to see if the input line is wiggling.  If so, the
;;; bootloader will synchronize with the PC and accept and burn the new
;;; program, or execute the commands.  Otherwise (no wiggling), the
;;; bootloader will jump to the application program.
;;;
;;; The bootloader will check for wiggling but, later, the application
;;; program *could* also check for wiggling if it is willing to allow
;;; a reprogramming of the application.
;;;
;;; The plan is to dedicate the RA5/Vpp/MCLR* pin to serial input.  If
;;; the application program needs this pin for another purpose, then
;;; the application program would not check for it wiggling, of course.
;;; 
;;; We also need a routine that will sit and wait until a character is
;;; received.
;;; 
;;; The bit period at 9600 bps is 104-1/6 uS
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Rx1
        ;; Since this is inverted TTL serial, a start/zero bit is a logic
        ;; high and a stop/one bit is a logic low.  Wait for a rising edge
        ;; and take that as the beginning of a start bit.  If the ninth bit
        ;; is not a stop bit, then we must be out of sync, so we could either
        ;; abandon the byte and wait for a new start bit or we could look
        ;; backward to the most earliest zero bit after what we had thought
        ;; was a start bit, take that zero bit as a start bit and continue.
        ;; Eventually, that should get us in sync.
myRx1
WaitForStop
        IfRxZero       ; make sure we have logic low before looking for start bit
        goto WaitForStop
WaitForStart
        IfRxOne
        goto WaitForStart
        ;; at this point, we have a rising edge that we hope is a start bit
        movlw 0x7F              ; plug in the start bit on the left
        movwf rxdata            ;  when it falls out, we've got our 8 bits
        KillMicroseconds 100     ; move to approximately the middle of the bit period
        IfRxOne                 ; if still a start bit (logic high) then
        goto WaitForStart       ;  all is well, else false alarm so go back
GetNextBit        
        call KillBitPeriodFor4800
GNB2        
        bcf STATUS, C           ; clear the carry flag, assuming new bit will be zero
        IfRxOne
        bsf STATUS, C           ; correct our assumption by setting the carry flag
RxShift
        rrf rxdata, F           ; shift new bit in at the left, as we shift right
        ;; if the start bit falls out, then we are done, we think
        IfC
        goto GetNextBit         ; no, a one fell out, so go back for another bit

        call KillBitPeriodFor4800 ; wait for what *should* be the stop bit
        IfRxZero
        goto GNB2               ; utoh, it wasn't a stop bit, so keep trying
                                ;   and maybe we will resynchronize eventually
        ;; ok, the stop bit occurred where expected, so we have our new character in rxdata
        ;; return it in W
        movf rxdata, W
        return
        
;;; Rx1
        ;; Since this is inverted TTL serial, a start/zero bit is a logic
        ;; high and a stop/one bit is a logic low.  Wait for a rising edge
        ;; and take that as the beginning of a start bit.  If the ninth bit
        ;; is not a stop bit, then we must be out of sync, so we could either
        ;; abandon the byte and wait for a new start bit or we could look
        ;; backward to the most earliest zero bit after what we had thought
        ;; was a start bit, take that zero bit as a start bit and continue.
        ;; Eventually, that should get us in sync.
	;; doautobaud looks for a start bit. It then counts bit widths recording
	;; The amount of time it takes to change. Let's store them for now
	;; Then transmit after.

doautobaud
	clrf	count	; clock count.
	movlw	cbuffer	; Set up the count buffer.
	movwf	FSR	; Point FSR to the buffer.
dWaitForStop
        IfRxZero       ; make sure we have logic low before looking for start bit
        goto dWaitForStop
dWaitForStart
        IfRxOne
        goto dWaitForStart
        ;; at this point, we have a rising edge that we hope is a start bit
	;; Start recording. We simply record pairs of edges. time out after
	;; 2 ms (bit 3 of TMR1H)

	clrf	TMR1L	; Reset the count.

        return
        
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; End of software serial input routine, (inverted TTL serial)
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        
;;; Variables
        ;; flash programming variables start at 0x70 so they
        ;;  available in all banks
        ;; ( on '819 could use 0x40-0x7F as those common to all 4 banks)
addrH  equ 0x70                  ; upper byte of Flash address
addrL  equ 0x71                  ; lower byte of Flash address
dataH  equ 0x72                  ; upper byte of Flash word
dataL  equ 0x73                  ; lower byte of Flash word
countH equ 0x74                  ; upper byte of counter (e.g. LForV macro)
countL equ 0x75                  ; lower byte of counter
delayH equ 0x76                  ; used by the Delay subroutine
delayL equ 0x77                  ;    and the KillMicroseconds macro

;;;; Bank0 variables
 cblock 0x20
  txdata      ; holds serial character being transmitted
  rxdata       ; holds character being collected from serial port
  lastSerialIn   ; remember state of serial input bit (in the RXPIN position) last
                 ;  time it was checked
  wiggles        ; count of how many wiggles have been detected
 endc

 end
