;----------------------------------------------------------------------
; Macros
;----------------------------------------------------------------------
Bank0	macro
	bcf	STATUS, RP0
	bcf	STATUS, RP1
 endm

Bank1	macro
	bsf	STATUS, RP0
	bcf	STATUS, RP1
 endm

Bank2	macro
	bcf	STATUS, RP0
	bsf	STATUS, RP1
 endm

Bank3	macro
	bsf	STATUS, RP0
	bsf	STATUS, RP1
 endm

        ;; following two are short versions when there is no
        ;; chance the previous bank was either 2 or 3.
Bnk0	macro
	bcf	STATUS, RP0
 endm

Bnk1	macro
	bsf	STATUS, RP0
 endm

IfZ	macro
	btfsc	STATUS, Z
 endm

IfNotZ	macro
	btfss	STATUS, Z
 endm

IfEq	macro	Var
	subwf	Var, W
	IfZ
 endm

IfNotEq	macro	Var
	subwf	Var, W
	IfNotZ
 endm

IfC	macro
	btfsc	STATUS, C
 endm

IfNotC	macro
	btfss	STATUS, C
 endm


IfBorrow macro
        IfNotC
 endm

IfNotBorrow macro
        IfC
 endm

;;; IfEqLit execute the following instruction if W equals the literal
IfEqLit macro val
        xorlw val
        IfZ
 endm

;;; IfNotEqLit execute the following instruction if W equals the literal
IfNotEqLit macro val
        xorlw val
        IfNotZ
 endm
        

	; If Less Than
        ;; read it as "If W is less than Var"
IfLT	macro	Var
	subwf	Var, W
	IfC
 endm

	; If Not Less Than (ie If Greater Than or Equal)
        ;; read it as "If W is not less than Var"
IfNotLT	macro	Var
	subwf	Var, W
	IfNotC
 endm

;;; Save the state of W and STATUS (mainly need in ISRs)
;;; To use this macro, be sure to create w_temp (in common
;;; memory) and s_temp (in bank 0 or in common memory) variables.
SaveWS	macro
	movwf	w_temp
	swapf	STATUS, W
        clrf    STATUS          ; bank 0, regardless of current bank
	movwf	s_temp
	endm

RestoreWS	macro
	swapf	s_temp, W
	movwf	STATUS
	swapf	w_temp, F
	swapf	w_temp, W
	endm

SaveFsr macro
	movf FSR, W
	movwf save_fsr
	endm
RestoreFsr macro
	movf save_fsr, W
	movwf FSR
	endm
;;;; For/Next  For/Next with counter in a register
        ;; For sets up a loop destination after rotating the 
        ;; 3-deep stack of destinations.
For     macro reg
        ifndef dest1
          variable dest1, dest2, dest3
        endif
dest3   set dest2
dest2   set dest1
dest1   set $
        endm
        ;; Next decrements register and jumps back to destination set by
        ;; closest previous For.  Then unrotate the destination stack.
Next    macro reg
        decfsz reg, F
        goto dest1
dest1   set dest2
dest2   set dest3                
        endm                                

;;; ;;;;;;;;;;;;;;;;;;;;;;;   LFor and LNext and LForV ;;;;;;;;;;;;;;;;;;;;;;;;
;;; *** Note, we could shorten the code by having the macro subtract the
;;; ***  the desired count from 0x10000 and then counting up.
;;; ***  Although that would complicate the LForV macro I guess.
;;; ***
;;; LFor takes a 16-bit count which will be put into the dedicated
;;;  counter.  We might have an alternate (named LForV perhaps) that
;;;  will take two registers rather than a literal count.
;;;  Note, LFor/LNext cannot be nested.
;;; LFor cnt cannot take a count of zero (as that is taken as 0x10000)
;;;  So, the minimum loop executions is 1.
LFor macro cnt
        ifndef lfordest
          variable lfordest
        endif
        ;; put (cnt - 1) into countH:countL 
cnt1    set cnt - 1             
        movlw high(cnt1)
        movwf countH
        movlw low(cnt1)
        movwf countL
        ;; set a variable for to address of the main body of the loop
lfordest set $        
        endm

;;; LForV is like LFor except the count is already in countH:countL
;;;  The same LNext will end either LForV or LFor -- do not nest
LForV macro
        ifndef lfordest
          variable lfordest
        endif
        movlw 1
        subwf countL, F         ; decrement countL and set flags
        IfBorrow
        decf countH, F
        ;; above puts (cnt - 1) into countH:countL 
        ;; set a variable for to address of the main body of the loop
lfordest set $        
        endm
        
;;; since we initialized the count to one less than the desired number
;;; of repetitions, the loop is finished when decrementing the counter
;;; causes it to reach -1  (0xFFFF : 0xFFFF)
LNext   macro
        movlw 1
        subwf countL, F         ; decrement countL and set flags
        IfNotBorrow             ; if C (ie no borrow) then we are not finished
        goto lfordest
                                ; otherwise
        subwf countH, F         ;  decrement countH also (W still contains 1)
        IfNotBorrow             ;  if C (i. no borrow) then we not finished,
        goto lfordest
        endm
;;; ;;;;;;;;;;;;;;;;;;;;;;;  end of LFor and LNext and LForV ;;;;;;;;;;;;;;;;;;;
