#!/usr/bin/env python2.6

# Communicate with PIC bootloader

import sys
import serial
import time
import threading

SYNC = 0xA5           # must be the same used in boot.asm

BLOCKDELAY = 0.005    # 5 milliseconds -- should work for '818, '819, '87x
BYTEDELAY =  0.00005    # we might reduce this to 0.002 or 0.003


def abortWithUsageMessage():
    msg = """
Usage:
      picbl <intelhexfile> <serialportnumber>
  e.g. to download the Intel hex file named 'dummy1.hex' via the
  first serial port (COM1: under DOS/Windows, /dev/ttyS0 under Linux)
  say:
      picbl dummy1.hex 0
"""
    print msg
    sys.exit(1)

def parseCommandLine():
    """Validate the command line and open the hex file and the serial port,
       setting global variables to refer to them.
    """
    global portNumber, hexFile, serialPort
    if len(sys.argv) < 2:
       portNumber = "/dev/ttyUSB0"
    else:
       try:
          portNumber = sys.argv[1]
       except:
          print "The port number %s should be an integer between 0 and 7" % sys.argv[2]
          abortWithUsageMessage()

    try:
        serialPort = openSerialPort(portNumber)
    except:
        print "Cannot open serial port number %d" % portNumber
        abortWithUsageMessage()
        
    print "Okey dokey, we'll opened the serial port %s" % (portNumber)

def openSerialPort(num):
    """Open the /dev/ttyS0 or /dev/ttyS1 etc, specifying everything
       even though most parameters default to what we want.
       'timeout=0' means non-blocking
    """
    ser = serial.Serial(port=num,
                        baudrate=115200,
                        bytesize=serial.EIGHTBITS,
                        parity=serial.PARITY_NONE,
                        stopbits=serial.STOPBITS_TWO,
                        timeout=0, xonxoff=0, rtscts=0,
                        writeTimeout=None)
    return ser


def serout (intval):
    """Convert the integer value to a character and transmit it via the
       serial port, then delay the appropriate delay.
    """
    ch = chr(intval)
    serialPort.write(ch)
    time.sleep (BYTEDELAY)

def seroutwait (intval):
    """Like serout() but wait for the character to be echo'd back
    """
    ch = chr(intval)
    serialPort.write(ch)
    while serialPort.read() != ch:
        pass
EX = 0
def echoIncoming():
    """To be run in a background thread.  Echo each serial input character.
       Start it somewhere with:
             t = threading.Thread(None, echoIncoming)
             t.start()
    """
    while EX == 0:
        c = serialPort.read()
        if c:
            # later we could handle crlf and non-printable characters
            sys.stdout.write(c)
            sys.stdout.flush()

def startEchoing():
    t = threading.Thread(None, echoIncoming)
    t.start()

def stop ():
    global EX
    EX = 1
    
def clockbit(bit):
   serialPort.setDTR(bit & 1)
   time.sleep(0.001);
   serialPort.write(chr(0));
   time.sleep(0.001);


def readbit():
   serialPort.write(chr(0));
   time.sleep (0.01)
   return 1 if serialPort.getCTS() else 0
   
def clockcmd(name,cmd):
#   print "Clocking", name, "command bits: ",
   mask = 1
   while mask < 0x0040:
      bit = 1 if mask & cmd else 0
      clockbit(bit);
#      print bit,
      mask = mask << 1
#   print
   time.sleep(0.00001);

def clockdata(data):
#   print "Clocking data bits: 0",
   mask = 1
   clockbit(0)
   while mask < 0x4000:
      bit = 1 if mask & data else 0
      clockbit(bit);
#      print bit,
      mask = mask << 1
#   print "0"
   clockbit(0)

def reset_part():
    serialPort.setDTR(0)
    serialPort.setRTS(0)
    cmd = raw_input('Remove Vdd, then press ENTER.');
    serialPort.setRTS(1)
    cmd = raw_input('Replace Vdd, then press ENTER.');

def read_word():
   clockcmd("read",0x0004)
#   time.sleep(0.001)
   data = 0
   count = 0
   while count < 16:
      data = (data >> 1) | (readbit() << 14) 
      count = count + 1
#   print "Read value: %(num)04x" % {"num":data}
   time.sleep(0.001)
   return data & 0x3fff

def get_id():
   clockcmd("Load config",0x0000)
   clockdata(0x0000)
   i=0

   while (i < 6):
      clockcmd("increment address",0x0006)
      i = i + 1

   my_id = read_word()
   print "ID value: %(num)04x" % {"num":my_id}

def load_data(data):
#   print "Loading data: %(data)04x ", {"data":data}
   clockcmd("data",0x0002)
   clockdata(data)
   time.sleep(0.00001)

def erase_all():
   clockcmd("Configuration",0x0000)
   clockdata(0x0000)
   clockcmd("Erase All",0x0009)
   time.sleep(0.008)

if __name__ == '__main__':
    parseCommandLine()
    dtrval = 0
    count = 0
    serialPort.setRTS(dtrval)
    serialPort.setDTR(dtrval)
    print "Set DTR(Data:7) and RTS(Vpp:4) to: ", dtrval
    reset_part()
    get_id()

    testlist = [ 0x0000,0x3fff,0x2C4C,0x2323,0x3f3f,0x0000,0x3fff ] 
    for num in testlist:
       print "Erasing Part"
       reset_part()
       erase_all()
       print "Programming part with %(num)04x" % {"num":num}
       reset_part()
       i = 0
       while i < 256:
          load_data(num)
          clockcmd("increment address",0x0006)
          load_data(num)
          clockcmd("increment address",0x0006)
          load_data(num)
          clockcmd("increment address",0x0006)
          load_data(num)
          clockcmd("Programming",0x0018)
          time.sleep(0.05)
          clockcmd("End Programming",0x000A)
          clockcmd("increment address",0x0006)
          i = i + 4
          if i & 31 == 0:
             print ".",
          if i & 1023 == 0:
             print "\n%(k)%d" % {"k":i / 1024},
          sys.stdout.flush();

       print
       reset_part()
       i = 0
       while i < 256:
          got = read_word()
          if got != num:
             print "\n%(addr)04x: Failed read %(got)04x != %(num)04x\n" % \
                {"addr":i,"got":got,"num":num}
          i = i+1 
          clockcmd("increment address",0x0006)
          time.sleep(0.01)
          if i & 31 == 0:
             print ".",
          if i & 1023 == 0:
             print "\n%(k)%d" % {"k":i / 1024},
          sys.stdout.flush();

       print

       
#    finished = 0
#    while finished == 0:
#       cmd = raw_input('cmd> ');
#       todo = cmd[0]
#       if todo == 'p':
#           if (len(cmd) > 1):
#              serialPort.setRTS(int(cmd[1]) & 1)
#              print "Set RTS(Vpp:4) to: ", int(cmd[1]) & 1
#           else:
#              print "command incomplete"
#       elif todo == 'd':
#           if (len(cmd) > 1):
#              serialPort.setDTR(int(cmd[1]) & 1)
#              print "Set DTR(Data:7) to: ", int(cmd[1]) & 1
#           else:
#              print "command incomplete"
#       elif todo == 'c':
#           print "Clocking TX(Clock:6) "
#	   serialPort.write(chr(0));
#       elif todo == 'B':
#           print "Clocking these bits: ",
#           for bit in cmd[len(cmd):0:-1]:
#               clockbit(int(bit));
#               print bit,
#           print
#       elif todo == 'C':
#           clockcmd("Load config",0x0000)
#           clockdata(0x0000)
#       elif todo == 'D':
#           print "Loading data: ", cmd[1:] 
#           clockcmd("data",0x0002)
#           clockdata(int(cmd[1:],16))
#       elif todo == 'I':
#           clockcmd("increment address",0x0006)
#       elif todo == 'r':
#           print "The current data value (6) is:",serialPort.getCTS()
#       elif todo == 'R':
#           clockcmd("read",0x0004)
#           data = 0
#           count = 0
#           while count < 16:
#              data = (data >> 1) | (readbit() << 14) 
#              count = count + 1
#           print "Read value: %(num)04x" % {"num":data}
#       elif todo == '?':
#           print "p<bit>        : set Vpp to bit"
#           print "d<bit>        : set data line to bit"
#           print "c             : blip clock line"
#           print "B<bit string> : clock bit string (LSB first)"
#           print "C             : Load Config"
#           print "D<hex data>   : Load Data"
#           print "I             : Increment address"
#           print "R             : Read Memory"
#           print "q             : quit"
#       elif todo == 'q':
#           print "Quitting."
#           finished = 1
#       else:
#           print "Unknown command."
#
# 
#    # comment out one or the other of the next two lines
#    #startEchoing()
#    serialPort.close()
#    
## Local Variables:
## py-master-file: nil
## End:
